@@ -50,11 +50,23 @@ G_BEGIN_DECLS
 #define DBUSMENU_MENUITEM_SIGNAL_REALIZED            "realized"
 #define DBUSMENU_MENUITEM_SIGNAL_REALIZED_ID         (g_signal_lookup(DBUSMENU_MENUITEM_SIGNAL_REALIZED, DBUSMENU_TYPE_MENUITEM))
 
+#define DBUSMENU_MENUITEM_PROP_TYPE                  "type"
 #define DBUSMENU_MENUITEM_PROP_VISIBLE               "visible"
-#define DBUSMENU_MENUITEM_PROP_SENSITIVE             "sensitive"
+#define DBUSMENU_MENUITEM_PROP_ENABLED               "enabled"
 #define DBUSMENU_MENUITEM_PROP_LABEL                 "label"
-#define DBUSMENU_MENUITEM_PROP_ICON                  "icon"
+#define DBUSMENU_MENUITEM_PROP_ICON_NAME             "icon-name"
 #define DBUSMENU_MENUITEM_PROP_ICON_DATA             "icon-data"
+#define DBUSMENU_MENUITEM_PROP_TOGGLE_TYPE           "toggle-type"
+#define DBUSMENU_MENUITEM_PROP_TOGGLE_STATE          "toggle-state"
+
+#define DBUSMENU_MENUITEM_TOGGLE_CHECK               "checkmark"
+#define DBUSMENU_MENUITEM_TOGGLE_RADIO               "radio"
+
+#define DBUSMENU_MENUITEM_TOGGLE_STATE_UNCHECKED     0
+#define DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED       1
+#define DBUSMENU_MENUITEM_TOGGLE_STATE_UNKNOWN       -1
+
+#define DBUSMENU_MENUITEM_ICON_NAME_BLANK            "blank-icon"
 
 /**
 	DbusmenuMenuitem:
@@ -72,6 +84,17 @@ struct _DbusmenuMenuitem
 	GObject parent;
 };
 
+/**
+	dbusmenu_menuitem_about_to_show_cb:
+	@mi Menu item that should be shown
+	@user_data Extra user data sent with the function
+
+	Callback prototype for a callback that is called when the
+	menu should be shown.
+*/
+typedef void (*dbusmenu_menuitem_about_to_show_cb) (DbusmenuMenuitem * mi, gpointer user_data);
+
+
 /**
 	DbusmenuMenuitemClass:
 	@property_changed: Slot for #DbusmenuMenuitem::property-changed.
@@ -82,6 +105,11 @@ struct _DbusmenuMenuitem
 	@realized: Slot for #DbusmenuMenuitem::realized.
 	@buildxml: Virtual function that appends the strings required
 	           to represent this menu item in the menu XML file.
+	@handle_event: This function is to override how events are handled
+			by subclasses.  Look at #dbusmenu_menuitem_handle_event for
+			lots of good information.
+	@send_about_to_show: Virtual function that notifies server that the
+			client is about to show a menu.
 	@reserved1: Reserved for future use.
 	@reserved2: Reserved for future use.
 	@reserved3: Reserved for future use.
@@ -93,8 +121,8 @@ struct _DbusmenuMenuitemClass
 	GObjectClass parent_class;
 
 	/* Signals */
-	void (*property_changed) (gchar * property, gchar * value);
-	void (*item_activated) (void);
+	void (*property_changed) (gchar * property, GValue * value);
+	void (*item_activated) (guint timestamp);
 	void (*child_added) (DbusmenuMenuitem * child, guint position);
 	void (*child_removed) (DbusmenuMenuitem * child);
 	void (*child_moved) (DbusmenuMenuitem * child, guint newpos, guint oldpos);
@@ -102,18 +130,20 @@ struct _DbusmenuMenuitemClass
 
 	/* Virtual functions */
 	void (*buildxml) (GPtrArray * stringarray);
+	void (*handle_event) (DbusmenuMenuitem * mi, const gchar * name, const GValue * value, guint timestamp);
+	void (*send_about_to_show) (DbusmenuMenuitem * mi, dbusmenu_menuitem_about_to_show_cb cb, gpointer cb_data);
 
 	void (*reserved1) (void);
-	void (*reserved2) (void);
-	void (*reserved3) (void);
+	/* void (*reserved2) (void); */
+	/* void (*reserved3) (void); */
 	/* void (*reserved4) (void); -- realized, realloc when bumping lib version */
 };
 
 GType dbusmenu_menuitem_get_type (void);
 
 DbusmenuMenuitem * dbusmenu_menuitem_new (void) G_GNUC_WARN_UNUSED_RESULT;
-DbusmenuMenuitem * dbusmenu_menuitem_new_with_id (guint id) G_GNUC_WARN_UNUSED_RESULT;
-guint dbusmenu_menuitem_get_id (DbusmenuMenuitem * mi);
+DbusmenuMenuitem * dbusmenu_menuitem_new_with_id (gint id) G_GNUC_WARN_UNUSED_RESULT;
+gint dbusmenu_menuitem_get_id (DbusmenuMenuitem * mi);
 
 GList * dbusmenu_menuitem_get_children (DbusmenuMenuitem * mi);
 GList * dbusmenu_menuitem_take_children (DbusmenuMenuitem * mi) G_GNUC_WARN_UNUSED_RESULT;
@@ -124,21 +154,28 @@ gboolean dbusmenu_menuitem_child_prepend (DbusmenuMenuitem * mi, DbusmenuMenuite
 gboolean dbusmenu_menuitem_child_delete (DbusmenuMenuitem * mi, DbusmenuMenuitem * child);
 gboolean dbusmenu_menuitem_child_add_position (DbusmenuMenuitem * mi, DbusmenuMenuitem * child, guint position);
 gboolean dbusmenu_menuitem_child_reorder (DbusmenuMenuitem * mi, DbusmenuMenuitem * child, guint position);
-DbusmenuMenuitem * dbusmenu_menuitem_child_find (DbusmenuMenuitem * mi, guint id);
-DbusmenuMenuitem * dbusmenu_menuitem_find_id (DbusmenuMenuitem * mi, guint id);
+DbusmenuMenuitem * dbusmenu_menuitem_child_find (DbusmenuMenuitem * mi, gint id);
+DbusmenuMenuitem * dbusmenu_menuitem_find_id (DbusmenuMenuitem * mi, gint id);
 
 gboolean dbusmenu_menuitem_property_set (DbusmenuMenuitem * mi, const gchar * property, const gchar * value);
+gboolean dbusmenu_menuitem_property_set_value (DbusmenuMenuitem * mi, const gchar * property, const GValue * value);
+gboolean dbusmenu_menuitem_property_set_bool (DbusmenuMenuitem * mi, const gchar * property, const gboolean value);
+gboolean dbusmenu_menuitem_property_set_int (DbusmenuMenuitem * mi, const gchar * property, const gint value);
 const gchar * dbusmenu_menuitem_property_get (DbusmenuMenuitem * mi, const gchar * property);
+const GValue * dbusmenu_menuitem_property_get_value (DbusmenuMenuitem * mi, const gchar * property);
+gboolean dbusmenu_menuitem_property_get_bool (DbusmenuMenuitem * mi, const gchar * property);
+gint dbusmenu_menuitem_property_get_int (DbusmenuMenuitem * mi, const gchar * property);
 gboolean dbusmenu_menuitem_property_exist (DbusmenuMenuitem * mi, const gchar * property);
 GList * dbusmenu_menuitem_properties_list (DbusmenuMenuitem * mi) G_GNUC_WARN_UNUSED_RESULT;
 GHashTable * dbusmenu_menuitem_properties_copy (DbusmenuMenuitem * mi);
+void dbusmenu_menuitem_property_remove (DbusmenuMenuitem * mi, const gchar * property);
 
 void dbusmenu_menuitem_set_root (DbusmenuMenuitem * mi, gboolean root);
 gboolean dbusmenu_menuitem_get_root (DbusmenuMenuitem * mi);
 
-void dbusmenu_menuitem_buildxml (DbusmenuMenuitem * mi, GPtrArray * array, gint revision);
 void dbusmenu_menuitem_foreach (DbusmenuMenuitem * mi, void (*func) (DbusmenuMenuitem * mi, gpointer data), gpointer data);
-void dbusmenu_menuitem_activate (DbusmenuMenuitem * mi);
+void dbusmenu_menuitem_handle_event (DbusmenuMenuitem * mi, const gchar * name, const GValue * value, guint timestamp);
+void dbusmenu_menuitem_send_about_to_show (DbusmenuMenuitem * mi, dbusmenu_menuitem_about_to_show_cb cb, gpointer cb_data);
 
 /**
 	SECTION:menuitem
