@@ -47,21 +47,21 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* VOID:UINT,STRING,STRING (./server-marshal.list:1) */
+/* VOID:INT,STRING,POINTER (./server-marshal.list:1) */
 void
-_dbusmenu_server_marshal_VOID__UINT_STRING_STRING (GClosure     *closure,
+_dbusmenu_server_marshal_VOID__INT_STRING_POINTER (GClosure     *closure,
                                                    GValue       *return_value G_GNUC_UNUSED,
                                                    guint         n_param_values,
                                                    const GValue *param_values,
                                                    gpointer      invocation_hint G_GNUC_UNUSED,
                                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__UINT_STRING_STRING) (gpointer     data1,
-                                                         guint        arg_1,
+  typedef void (*GMarshalFunc_VOID__INT_STRING_POINTER) (gpointer     data1,
+                                                         gint         arg_1,
                                                          gpointer     arg_2,
                                                          gpointer     arg_3,
                                                          gpointer     data2);
-  register GMarshalFunc_VOID__UINT_STRING_STRING callback;
+  register GMarshalFunc_VOID__INT_STRING_POINTER callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -77,12 +77,49 @@ _dbusmenu_server_marshal_VOID__UINT_STRING_STRING (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__UINT_STRING_STRING) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__INT_STRING_POINTER) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_int (param_values + 1),
             g_marshal_value_peek_string (param_values + 2),
-            g_marshal_value_peek_string (param_values + 3),
+            g_marshal_value_peek_pointer (param_values + 3),
+            data2);
+}
+
+/* VOID:UINT,INT (./server-marshal.list:2) */
+void
+_dbusmenu_server_marshal_VOID__UINT_INT (GClosure     *closure,
+                                         GValue       *return_value G_GNUC_UNUSED,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint G_GNUC_UNUSED,
+                                         gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT_INT) (gpointer     data1,
+                                               guint        arg_1,
+                                               gint         arg_2,
+                                               gpointer     data2);
+  register GMarshalFunc_VOID__UINT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
             data2);
 }
 
