@@ -30,16 +30,23 @@ License version 3 and version 2.1 along with this program.  If not, see
 #include "config.h"
 #endif
 
+#include "menuitem-private.h"
 #include "server.h"
 #include "server-marshal.h"
 
 /* DBus Prototypes */
-static gboolean _dbusmenu_server_get_property (DbusmenuServer * server, guint id, gchar * property, gchar ** value, GError ** error);
-static gboolean _dbusmenu_server_get_properties (DbusmenuServer * server, guint id, GHashTable ** dict, GError ** error);
-static gboolean _dbusmenu_server_call (DbusmenuServer * server, guint id, GError ** error);
+static gboolean _dbusmenu_server_get_layout (DbusmenuServer * server, gint parent, guint * revision, gchar ** layout, GError ** error);
+static gboolean _dbusmenu_server_get_property (DbusmenuServer * server, gint id, gchar * property, gchar ** value, GError ** error);
+static gboolean _dbusmenu_server_get_properties (DbusmenuServer * server, gint id, GPtrArray * properties, GHashTable ** dict, GError ** error);
+static gboolean _dbusmenu_server_get_group_properties (DbusmenuServer * server, GArray * ids, GArray * properties, GHashTable ** values, GError ** error);
+static gboolean _dbusmenu_server_event (DbusmenuServer * server, gint id, gchar * eventid, GValue * data, guint timestamp, GError ** error);
+static gboolean _dbusmenu_server_get_children (DbusmenuServer * server, gint id, GPtrArray * properties, GPtrArray ** output, GError ** error);
+static gboolean _dbusmenu_server_about_to_show (DbusmenuServer * server, gint id, gboolean * need_update, GError ** error);
 
 #include "dbusmenu-server.h"
 
+#define DBUSMENU_VERSION_NUMBER  2
+
 /* Privates, I'll show you mine... */
 typedef struct _DbusmenuServerPrivate DbusmenuServerPrivate;
 
@@ -57,7 +64,7 @@ struct _DbusmenuServerPrivate
 enum {
 	ID_PROP_UPDATE,
 	ID_UPDATE,
-	LAYOUT_UPDATE,
+	LAYOUT_UPDATED,
 	LAST_SIGNAL
 };
 
@@ -68,7 +75,7 @@ enum {
 	PROP_0,
 	PROP_DBUS_OBJECT,
 	PROP_ROOT_NODE,
-	PROP_LAYOUT
+	PROP_VERSION
 };
 
 /* Errors */
@@ -76,6 +83,7 @@ enum {
 	INVALID_MENUITEM_ID,
 	INVALID_PROPERTY_NAME,
 	UNKNOWN_DBUS_ERROR,
+	NOT_IMPLEMENTED,
 	LAST_ERROR
 };
 
@@ -86,7 +94,7 @@ static void dbusmenu_server_dispose    (GObject *object);
 static void dbusmenu_server_finalize   (GObject *object);
 static void set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec);
 static void get_property (GObject * obj, guint id, GValue * value, GParamSpec * pspec);
-static void menuitem_property_changed (DbusmenuMenuitem * mi, gchar * property, gchar * value, DbusmenuServer * server);
+static void menuitem_property_changed (DbusmenuMenuitem * mi, gchar * property, GValue * value, DbusmenuServer * server);
 static void menuitem_child_added (DbusmenuMenuitem * parent, DbusmenuMenuitem * child, guint pos, DbusmenuServer * server);
 static void menuitem_child_removed (DbusmenuMenuitem * parent, DbusmenuMenuitem * child, DbusmenuServer * server);
 static void menuitem_signals_create (DbusmenuMenuitem * mi, gpointer data);
@@ -122,8 +130,8 @@ dbusmenu_server_class_init (DbusmenuServerClass *class)
 	                                         G_SIGNAL_RUN_LAST,
 	                                         G_STRUCT_OFFSET(DbusmenuServerClass, id_prop_update),
 	                                         NULL, NULL,
-	                                         _dbusmenu_server_marshal_VOID__UINT_STRING_STRING,
-	                                         G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING);
+	                                         _dbusmenu_server_marshal_VOID__INT_STRING_POINTER,
+	                                         G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_STRING, G_TYPE_VALUE);
 	/**
 		DbusmenuServer::id-update:
 		@arg0: The #DbusmenuServer emitting the signal.
@@ -141,37 +149,38 @@ dbusmenu_server_class_init (DbusmenuServerClass *class)
 	                                         g_cclosure_marshal_VOID__UINT,
 	                                         G_TYPE_NONE, 1, G_TYPE_UINT);
 	/**
-		DbusmenuServer::layout-update:
+		DbusmenuServer::layout-updated:
 		@arg0: The #DbusmenuServer emitting the signal.
 		@arg1: A revision number representing which revision the update
 		       represents itself as.
+		@arg2: The ID of the parent for this update.
 
 		This signal is emitted any time the layout of the
 		menuitems under this server is changed.
 	*/
-	signals[LAYOUT_UPDATE] =    g_signal_new(DBUSMENU_SERVER_SIGNAL_LAYOUT_UPDATE,
+	signals[LAYOUT_UPDATED] =   g_signal_new(DBUSMENU_SERVER_SIGNAL_LAYOUT_UPDATED,
 	                                         G_TYPE_FROM_CLASS(class),
 	                                         G_SIGNAL_RUN_LAST,
-	                                         G_STRUCT_OFFSET(DbusmenuServerClass, layout_update),
+	                                         G_STRUCT_OFFSET(DbusmenuServerClass, layout_updated),
 	                                         NULL, NULL,
-	                                         g_cclosure_marshal_VOID__INT,
-	                                         G_TYPE_NONE, 1, G_TYPE_INT);
+	                                         _dbusmenu_server_marshal_VOID__UINT_INT,
+	                                         G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_INT);
 
 
 	g_object_class_install_property (object_class, PROP_DBUS_OBJECT,
 	                                 g_param_spec_string(DBUSMENU_SERVER_PROP_DBUS_OBJECT, "DBus object path",
 	                                              "The object that represents this set of menus on DBus",
-	                                              "/org/freedesktop/dbusmenu",
+	                                              "/org/ayatana/dbusmenu",
 	                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (object_class, PROP_ROOT_NODE,
 	                                 g_param_spec_object(DBUSMENU_SERVER_PROP_ROOT_NODE, "Root menu node",
 	                                              "The base object of the menus that are served",
 	                                              DBUSMENU_TYPE_MENUITEM,
 	                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (object_class, PROP_LAYOUT,
-	                                 g_param_spec_string(DBUSMENU_SERVER_PROP_LAYOUT, "XML Layout of the menus",
-	                                              "A simple XML string that describes the layout of the menus",
-	                                              "<menu />",
+	g_object_class_install_property (object_class, PROP_VERSION,
+	                                 g_param_spec_uint(DBUSMENU_SERVER_PROP_VERSION, "Dbusmenu API version",
+	                                              "The version of the DBusmenu API that we're implementing.",
+	                                              DBUSMENU_VERSION_NUMBER, DBUSMENU_VERSION_NUMBER, DBUSMENU_VERSION_NUMBER,
 	                                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	dbus_g_object_type_install_info(DBUSMENU_TYPE_SERVER, &dbus_glib__dbusmenu_server_object_info);
@@ -229,22 +238,21 @@ set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec)
 	case PROP_ROOT_NODE:
 		if (priv->root != NULL) {
 			dbusmenu_menuitem_foreach(priv->root, menuitem_signals_remove, obj);
+			dbusmenu_menuitem_set_root(priv->root, FALSE);
 			g_object_unref(G_OBJECT(priv->root));
 			priv->root = NULL;
 		}
 		priv->root = DBUSMENU_MENUITEM(g_value_get_object(value));
 		if (priv->root != NULL) {
 			g_object_ref(G_OBJECT(priv->root));
+			dbusmenu_menuitem_set_root(priv->root, TRUE);
 			dbusmenu_menuitem_foreach(priv->root, menuitem_signals_create, obj);
 		} else {
 			g_debug("Setting root node to NULL");
 		}
 		priv->layout_revision++;
-		g_signal_emit(obj, signals[LAYOUT_UPDATE], 0, priv->layout_revision, TRUE);
+		g_signal_emit(obj, signals[LAYOUT_UPDATED], 0, priv->layout_revision, 0, TRUE);
 		break;
-	case PROP_LAYOUT:
-		/* Can't set this, fall through to error */
-		g_warning("Can not set property: layout");
 	default:
 		g_return_if_reached();
 		break;
@@ -276,25 +284,9 @@ get_property (GObject * obj, guint id, GValue * value, GParamSpec * pspec)
 	case PROP_ROOT_NODE:
 		g_value_set_object(value, priv->root);
 		break;
-	case PROP_LAYOUT: {
-		GPtrArray * xmlarray = g_ptr_array_new();
-		if (priv->root == NULL) {
-			/* g_debug("Getting layout without root node!"); */
-			g_ptr_array_add(xmlarray, g_strdup_printf("<menu revision=\"%d\" />", priv->layout_revision));
-		} else {
-			dbusmenu_menuitem_buildxml(priv->root, xmlarray, priv->layout_revision);
-		}
-		g_ptr_array_add(xmlarray, NULL);
-
-		/* build string */
-		gchar * finalstring = g_strjoinv("", (gchar **)xmlarray->pdata);
-		g_value_take_string(value, finalstring);
-		/* g_debug("Final string: %s", finalstring); */
-
-		g_ptr_array_foreach(xmlarray, xmlarray_foreach_free, NULL);
-		g_ptr_array_free(xmlarray, TRUE);
+	case PROP_VERSION:
+		g_value_set_uint(value, DBUSMENU_VERSION_NUMBER);
 		break;
-	}
 	default:
 		g_return_if_reached();
 		break;
@@ -304,20 +296,39 @@ get_property (GObject * obj, guint id, GValue * value, GParamSpec * pspec)
 }
 
 static void 
-menuitem_property_changed (DbusmenuMenuitem * mi, gchar * property, gchar * value, DbusmenuServer * server)
+menuitem_property_changed (DbusmenuMenuitem * mi, gchar * property, GValue * value, DbusmenuServer * server)
 {
 	g_signal_emit(G_OBJECT(server), signals[ID_PROP_UPDATE], 0, dbusmenu_menuitem_get_id(mi), property, value, TRUE);
 	return;
 }
 
+/* Adds the signals for this entry to the list and looks at
+   the children of this entry to add the signals we need
+   as well.  We like signals. */
+static void
+added_check_children (gpointer data, gpointer user_data)
+{
+	DbusmenuMenuitem * mi = (DbusmenuMenuitem *)data;
+	DbusmenuServer * server = (DbusmenuServer *)user_data;
+
+	menuitem_signals_create(mi, server);
+	g_list_foreach(dbusmenu_menuitem_get_children(mi), added_check_children, server);
+
+	return;
+}
+
+/* Callback for when a child is added.  We need to connect everything
+   up and signal that the layout has changed. */
 static void
 menuitem_child_added (DbusmenuMenuitem * parent, DbusmenuMenuitem * child, guint pos, DbusmenuServer * server)
 {
 	menuitem_signals_create(child, server);
+	g_list_foreach(dbusmenu_menuitem_get_children(child), added_check_children, server);
+
 	/* TODO: We probably need to group the layout update signals to make the number more reasonble. */
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
 	priv->layout_revision++;
-	g_signal_emit(G_OBJECT(server), signals[LAYOUT_UPDATE], 0, priv->layout_revision, TRUE);
+	g_signal_emit(G_OBJECT(server), signals[LAYOUT_UPDATED], 0, priv->layout_revision, 0, TRUE);
 	return;
 }
 
@@ -328,7 +339,7 @@ menuitem_child_removed (DbusmenuMenuitem * parent, DbusmenuMenuitem * child, Dbu
 	/* TODO: We probably need to group the layout update signals to make the number more reasonble. */
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
 	priv->layout_revision++;
-	g_signal_emit(G_OBJECT(server), signals[LAYOUT_UPDATE], 0, priv->layout_revision, TRUE);
+	g_signal_emit(G_OBJECT(server), signals[LAYOUT_UPDATED], 0, priv->layout_revision, 0, TRUE);
 	return;
 }
 
@@ -337,7 +348,7 @@ menuitem_child_moved (DbusmenuMenuitem * parent, DbusmenuMenuitem * child, guint
 {
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
 	priv->layout_revision++;
-	g_signal_emit(G_OBJECT(server), signals[LAYOUT_UPDATE], 0, priv->layout_revision, TRUE);
+	g_signal_emit(G_OBJECT(server), signals[LAYOUT_UPDATED], 0, priv->layout_revision, 0, TRUE);
 	return;
 }
 
@@ -376,8 +387,38 @@ error_quark (void)
 }
 
 /* DBus interface */
+static gboolean
+_dbusmenu_server_get_layout (DbusmenuServer * server, gint parent, guint * revision, gchar ** layout, GError ** error)
+{
+	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
+
+	*revision = priv->layout_revision;
+	GPtrArray * xmlarray = g_ptr_array_new();
+
+	if (parent == 0) {
+		if (priv->root == NULL) {
+			/* g_debug("Getting layout without root node!"); */
+			g_ptr_array_add(xmlarray, g_strdup("<menu id=\"0\"/>"));
+		} else {
+			dbusmenu_menuitem_buildxml(priv->root, xmlarray);
+		}
+	} else {
+		DbusmenuMenuitem * item = dbusmenu_menuitem_find_id(priv->root, parent);
+		dbusmenu_menuitem_buildxml(item, xmlarray);
+	}
+	g_ptr_array_add(xmlarray, NULL);
+
+	/* build string */
+	*layout = g_strjoinv("", (gchar **)xmlarray->pdata);
+
+	g_ptr_array_foreach(xmlarray, xmlarray_foreach_free, NULL);
+	g_ptr_array_free(xmlarray, TRUE);
+
+	return TRUE;
+}
+
 static gboolean 
-_dbusmenu_server_get_property (DbusmenuServer * server, guint id, gchar * property, gchar ** value, GError ** error)
+_dbusmenu_server_get_property (DbusmenuServer * server, gint id, gchar * property, gchar ** value, GError ** error)
 {
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
 	DbusmenuMenuitem * mi = dbusmenu_menuitem_find_id(priv->root, id);
@@ -422,7 +463,7 @@ _dbusmenu_server_get_property (DbusmenuServer * server, guint id, gchar * proper
 }
 
 static gboolean
-_dbusmenu_server_get_properties (DbusmenuServer * server, guint id, GHashTable ** dict, GError ** error)
+_dbusmenu_server_get_properties (DbusmenuServer * server, gint id, GPtrArray * properties, GHashTable ** dict, GError ** error)
 {
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
 	DbusmenuMenuitem * mi = dbusmenu_menuitem_find_id(priv->root, id);
@@ -444,7 +485,57 @@ _dbusmenu_server_get_properties (DbusmenuServer * server, guint id, GHashTable *
 }
 
 static gboolean
-_dbusmenu_server_call (DbusmenuServer * server, guint id, GError ** error)
+_dbusmenu_server_get_group_properties (DbusmenuServer * server, GArray * ids, GArray * properties, GHashTable ** values, GError ** error)
+{
+	if (error != NULL) {
+		g_set_error(error,
+					error_quark(),
+					NOT_IMPLEMENTED,
+					"The GetGroupProperties function is not implemented, sorry.");
+	}
+	return FALSE;
+}
+
+static void
+_gvalue_array_append_int(GValueArray *array, gint i)
+{
+	GValue value = {0};
+
+	g_value_init(&value, G_TYPE_INT);
+	g_value_set_int(&value, i);
+	g_value_array_append(array, &value);
+	g_value_unset(&value);
+}
+
+static void
+_gvalue_array_append_hashtable(GValueArray *array, GHashTable * dict)
+{
+	GValue value = {0};
+
+	g_value_init(&value, dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE));
+	g_value_set_boxed(&value, dict);
+	g_value_array_append(array, &value);
+	g_value_unset(&value);
+}
+
+static void
+serialize_menuitem(gpointer data, gpointer user_data)
+{
+	DbusmenuMenuitem * mi = DBUSMENU_MENUITEM(data);
+	GPtrArray * output = (GPtrArray *)(user_data);
+
+	gint id = dbusmenu_menuitem_get_id(mi);
+	GHashTable * dict = dbusmenu_menuitem_properties_copy(mi);
+
+	GValueArray * item = g_value_array_new(1);
+	_gvalue_array_append_int(item, id);
+	_gvalue_array_append_hashtable(item, dict);
+
+	g_ptr_array_add(output, item);
+}
+
+static gboolean
+_dbusmenu_server_get_children (DbusmenuServer * server, gint id, GPtrArray * properties, GPtrArray ** output, GError ** error)
 {
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
 	DbusmenuMenuitem * mi = dbusmenu_menuitem_find_id(priv->root, id);
@@ -460,7 +551,54 @@ _dbusmenu_server_call (DbusmenuServer * server, guint id, GError ** error)
 		return FALSE;
 	}
 
-	dbusmenu_menuitem_activate(mi);
+	*output = g_ptr_array_new();
+	GList * children = dbusmenu_menuitem_get_children(mi);
+	g_list_foreach(children, serialize_menuitem, *output);
+
+	return TRUE;
+}
+
+static gboolean
+_dbusmenu_server_event (DbusmenuServer * server, gint id, gchar * eventid, GValue * data, guint timestamp, GError ** error)
+{
+	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
+	DbusmenuMenuitem * mi = dbusmenu_menuitem_find_id(priv->root, id);
+
+	if (mi == NULL) {
+		if (error != NULL) {
+			g_set_error(error,
+			            error_quark(),
+			            INVALID_MENUITEM_ID,
+			            "The ID supplied %d does not refer to a menu item we have",
+			            id);
+		}
+		return FALSE;
+	}
+
+	dbusmenu_menuitem_handle_event(mi, eventid, data, timestamp);
+	return TRUE;
+}
+
+/* Recieve the About To Show function.  Pass it to our menu item. */
+static gboolean
+_dbusmenu_server_about_to_show (DbusmenuServer * server, gint id, gboolean * need_update, GError ** error)
+{
+	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(server);
+	DbusmenuMenuitem * mi = dbusmenu_menuitem_find_id(priv->root, id);
+
+	if (mi == NULL) {
+		if (error != NULL) {
+			g_set_error(error,
+			            error_quark(),
+			            INVALID_MENUITEM_ID,
+			            "The ID supplied %d does not refer to a menu item we have",
+			            id);
+		}
+		return FALSE;
+	}
+
+	/* GTK+ does not support about-to-show concept for now */
+	*need_update = FALSE;
 	return TRUE;
 }
 
@@ -472,7 +610,7 @@ _dbusmenu_server_call (DbusmenuServer * server, guint id, GError ** error)
 
 	Creates a new #DbusmenuServer object with a specific object
 	path on DBus.  If @object is set to NULL the default object
-	name of "/org/freedesktop/dbusmenu" will be used.
+	name of "/org/ayatana/dbusmenu" will be used.
 
 	Return value: A brand new #DbusmenuServer
 */
@@ -480,7 +618,7 @@ DbusmenuServer *
 dbusmenu_server_new (const gchar * object)
 {
 	if (object == NULL) {
-		object = "/org/freedesktop/dbusmenu";
+		object = "/org/ayatana/dbusmenu";
 	}
 
 	DbusmenuServer * self = g_object_new(DBUSMENU_TYPE_SERVER,
